﻿using System;
using System.Threading;
using System.Diagnostics;
using SimNIS.Properties;
using System.Net;

namespace SimNIS
{
    public class CoreEventArgs : EventArgs
    {
        public int Code { get; set; }
        public const int SuccessCode = 8;
        public const int SpinnerCode = 16;
        public const int TimeoutCode = 32;
    }

    public class CoreLogic
    {
        private static CoreLogic instance = null;
        private CoreLogic() { }

        public static CoreLogic Instance
        {
            get
            {
                if (instance == null)
                {
                    instance = new CoreLogic();
                }
                return instance;
            }
        }
        
        public enum Status { Idle, Connecting, Success, Error, Timeout}
        public Status currentStatus = Status.Idle;
        public string address;
        public int timeout, request, success;
        public long startTime, currentTime;

        public event EventHandler<CoreEventArgs> PingSuccess;
        public event EventHandler<CoreEventArgs> SpinnerLaunch;
        public event EventHandler<CoreEventArgs> ErrorTimeout;


        public void InitializeLogic()
        {
            StartConnecting();

            while (currentStatus != Status.Idle)
            {
                HttpWebRequest req = (HttpWebRequest)WebRequest.Create(address);
                req.AllowAutoRedirect = false;
                req.Timeout = 500;

                try
                {
                    using (HttpWebResponse response = (HttpWebResponse)req.GetResponse())
                    {
                        startTime = DateTimeOffset.Now.ToUnixTimeSeconds();
                        request = 0;

                        if (currentStatus != Status.Success)
                        {
                            Debug.WriteLine("\nSUCCESS.");
                            OnPingSuccess();
                        }
                    }
                }
                catch (Exception ex)
                {
                    Debug.WriteLine("(" + request + ") NO SUCCESS...");
                    if (currentStatus != Status.Timeout)
                    {
                        if (currentStatus != Status.Error)
                        {
                            if (request > 2)
                            {
                                OnSpinnerLaunch();
                                currentStatus = Status.Error;
                            }
                        }
                        else
                        {
                            currentTime = DateTimeOffset.Now.ToUnixTimeSeconds();
                            if (currentTime - startTime > timeout)
                            {
                                OnErrorTimeout();
                            }
                        }
                    }
                    request++;
                }
                Thread.Sleep(3000);
            }
        }







        protected virtual void OnPingSuccess()
        {
            if (PingSuccess != null)
            {
                PingSuccess(this, new CoreEventArgs() { Code = CoreEventArgs.SuccessCode });
                currentStatus = Status.Success;
            }
        }

        
        protected virtual void OnSpinnerLaunch()
        {
            if (SpinnerLaunch != null)
            {
                SpinnerLaunch(this, new CoreEventArgs() { Code = CoreEventArgs.SpinnerCode });
            }
        }


        protected virtual void OnErrorTimeout()
        {
            if (ErrorTimeout != null)
            {
                ErrorTimeout(this, new CoreEventArgs() { Code = CoreEventArgs.TimeoutCode });
                currentStatus = Status.Timeout;
            }
        }


        public void StartConnecting()
        {
            OnSpinnerLaunch();
            startTime = DateTimeOffset.Now.ToUnixTimeSeconds();
            currentStatus = Status.Connecting;
            request = 0;
        }


        



    }
}

