﻿using System.IO;
using System.Diagnostics;
using SimNIS.Properties;
using System.Threading;
using System.Threading.Tasks;
using System;
using System.Windows;
using System.Collections.Generic;
using System.Text;
using System.Text.Json;
using System.Windows.Controls;

namespace SimNIS
{
    public class Script
    {
        public List<InfosData> tstInfos { get; set; }
        public List<ActionsData> actions { get; set; }
    }

    public class InfosData
    {
        public string name { get; set; }
        public string type { get; set; }
        public string description { get; set; }
    }

    public class ActionsData
    {
        public string id { get; set; }
        public string cmdMsg { get; set; }
        public int check { get; set; }
        public string setMsg { get; set; }
        public int sleep { get; set; }
        public string click { get; set; }
        public string url { get; set; }
        public int timeout { get; set; }
    }

    public class TestLogic
    {
        private static TestLogic instance = null;
        private TestLogic() { }

        public static TestLogic Instance
        {
            get
            {
                if (instance == null)
                {
                    instance = new TestLogic();
                }
                return instance;
            }
        }

        CoreLogic CoreLogic = CoreLogic.Instance;
        private int nextResult = 0;
        private bool resumeTest = true;
        public double startTime, endTime;
        List<string> logs = new List<string>();
        private static AutoResetEvent OnEventCheck = new AutoResetEvent(false);
        private enum EventMsg { Success, Spinner, Timeout }
        private EventMsg currentEventMsg;
        private enum ResultMsg { OK, NOOK }
        private ResultMsg currentResultMsg;
        private static AutoResetEvent OnStringBuild = new AutoResetEvent(false);
        public string cmdString = "", tName = "", tDescription = "", tType = "";
        public int currentClickX = 10000, currentClickY = 10000;
        private static AutoResetEvent OnClickCheck = new AutoResetEvent(false);
        private string nextUrl;
        private int outputCount = 0;



        public void RegisterToCoreLogic(bool active)
        {
            if (active)
            {
                CoreLogic.PingSuccess += OnReceivingCoreEvent;
                CoreLogic.SpinnerLaunch += OnReceivingCoreEvent;
                CoreLogic.ErrorTimeout += OnReceivingCoreEvent;
            }
            else
            {
                CoreLogic.PingSuccess -= OnReceivingCoreEvent;
                CoreLogic.SpinnerLaunch -= OnReceivingCoreEvent;
                CoreLogic.ErrorTimeout -= OnReceivingCoreEvent;
            }
        }


        public async void RunScript()
        {
            string filePath = Settings.Default.filePath;
            resumeTest = true;
            nextResult = 0;
            startTime = DateTimeOffset.Now.ToUnixTimeMilliseconds();
            outputCount = 0;
            currentClickTimeout = 0;
            UpdateOutputTextBox("INITIALIZING TEST");

            DoJsonConversion();
            OnStringBuild.WaitOne();

            StringReader sr = new StringReader(cmdString);
            while (true && resumeTest)
            {
                string line = sr.ReadLine();
                if (line != null)
                {
                    string[] words = line.Split('>');
                    switch (words[0])
                    {
                        case "CMD":
                            {
                                if (MainWindow.AppWindow.execOn == "PCM")
                                {
                                    UpdateOutputTextBox("SENDING COMMAND TO PCM...");
                                    await Task.Run(() => EstablishConnectionToPCM(words[1])); //To use with PCM
                                }
                                if (MainWindow.AppWindow.execOn == "CMD")
                                {
                                    await Task.Run(() => EstablishConnectionToCMD(words[1])); //To use at home
                                }
                                break;
                            }
                        case "CHECK":
                            {
                                if (words[1] == "url")
                                {
                                    if (currentClickX != 10000 && currentClickY != 10000)
                                    {
                                        if (words[2] == "")
                                        {
                                            MainWindow.AppWindow.MouseClick(currentClickX, currentClickY);
                                        }
                                        else
                                        {
                                            nextUrl = words[2];
                                            MainWindow.AppWindow.CheckAddress(true);
                                            MainWindow.AppWindow.ReceivingHttpStatusCode += OnReceivingClickEvent;
                                            DoClickLoop();
                                            OnClickCheck.WaitOne();
                                        }
                                    }
                                }
                                else
                                {
                                    UpdateOutputTextBox("Waiting next CoreLogic Event...");
                                    nextResult = Convert.ToInt32(words[1]);
                                    OnEventCheck.WaitOne();
                                    nextResult = 0;
                                }
                                break;
                            }
                        case "SET":
                            {
                                if (words[1] == "reconnection")
                                {
                                    CoreLogic.StartConnecting();
                                }
                                if (words[1] == "localhost")
                                {
                                    Application.Current.Dispatcher.Invoke(() =>
                                    {
                                        MainWindow.AppWindow.addressBar.Text = "http://localhost";
                                        MainWindow.AppWindow.SaveSettings();
                                    });
                                }
                                if (words[1] == "ip")
                                {
                                    Application.Current.Dispatcher.Invoke(() =>
                                    {
                                        MainWindow.AppWindow.addressBar.Text = words[2];
                                        MainWindow.AppWindow.SaveSettings();
                                    });
                                }
                                if (words[1] == "vehicleapp")
                                {
                                    Application.Current.Dispatcher.Invoke(() =>
                                    {
                                        MainWindow.AppWindow.VehicleTabItem.IsSelected = true;
                                    });
                                }
                                if (words[1] == "httpmonitoring")
                                {
                                    UpdateOutputTextBox("Http Status Codes monitoring ACTIVATED");
                                    MainWindow.AppWindow.ReceivingHttpStatusCode += OnReceivingHttpEvent;
                                }
                                break;
                            }
                        case "SLEEP":
                            {
                                Debug.WriteLine("WAITING FOR " + $"{words[1]}" + " SECONDS...");
                                UpdateOutputTextBox("WAITING FOR " + $"{words[1]}" + " SECONDS...");

                                int pause = Convert.ToInt32(words[1]) * 1000;
                                Task.WaitAll(new Task[] { Task.Delay(pause) });
                                break;
                            }
                        case "CLICK":
                            {
                                string[] coordinates = words[1].Split(',');
                                currentClickX = Convert.ToInt32(coordinates[0]);
                                currentClickY = Convert.ToInt32(coordinates[1]);
                                urlMatch = false;
                                break;
                            }
                        case "TIMEOUT":
                            {
                                currentClickTimeout = Convert.ToInt32(words[1]) * 1000;
                                break;
                            }
                    }
                }
                else
                {
                    break;
                }
            }
            sr.Close();
            endTime = DateTimeOffset.Now.ToUnixTimeMilliseconds();

            CreateOutputFile();
        }


        public void UpdateOutputTextBox(string output)
        {
            Task.WaitAll(new Task[] { Task.Delay(1000) });
            Application.Current.Dispatcher.Invoke(() =>
            {
                string lines = AutomatedTestWindow.TestWindow.OutputTextBox.Text;
                StringBuilder sb = new StringBuilder();
                sb.Append(lines).AppendLine().Append($"{outputCount}) " + output);
                AutomatedTestWindow.TestWindow.OutputTextBox.Text = sb.ToString();
                outputCount++;
            });
        }

        public async Task EstablishConnectionToPCM(string strCmdText)
        {
            string username = "root";
            string host = "192.168.249.1";
            string password = "root";

            ProcessStartInfo psi = new ProcessStartInfo()
            {
                FileName = Directory.GetCurrentDirectory() + $@"\plink.exe",
                Arguments = String.Format("-ssh {0}@{1} -pw {2}", username, host, password),
                RedirectStandardError = true,
                RedirectStandardOutput = true,
                RedirectStandardInput = true,
                UseShellExecute = false,
                CreateNoWindow = true
            };

            Process p = Process.Start(psi);

            StreamWriter strw = p.StandardInput;

            strw.WriteLine(strCmdText);
            strw.WriteLine("exit");

            p.WaitForExit();

            if (p.HasExited)
            {
                p.Close();
                p.Dispose();
            }
        }


        public async Task EstablishConnectionToCMD(string strCmdText)
        {
            ProcessStartInfo cmd = new ProcessStartInfo();
            cmd.UseShellExecute = true;
            cmd.WorkingDirectory = @"C:\Windows\System32";
            cmd.FileName = @"C:\Windows\System32\cmd.exe";
            cmd.Arguments = "/c " + strCmdText;
            cmd.WindowStyle = ProcessWindowStyle.Hidden;
            Process process = Process.Start(cmd);
            process.WaitForExit();

            if (process.HasExited)
            {
                process.Close();
                process.Dispose();
            }
        }


        public void DoJsonConversion()
        {
            UpdateOutputTextBox("READING JSON FILE SCRIPT...");

            string fileName = Settings.Default.filePath;
            if (fileName != "null")
            {
                string jsonString = File.ReadAllText(fileName);
                Script scriptData = JsonSerializer.Deserialize<Script>(jsonString);
                StringBuilder sb = new StringBuilder();
                foreach (var action in scriptData.actions)
                {
                    string setMsg = action.setMsg;
                    string cmdMsg = action.cmdMsg;
                    string check = action.check.ToString();
                    string sleep = action.sleep.ToString();
                    string click = action.click;
                    string url = action.url;
                    string timeout = action.timeout.ToString();

                    if (setMsg != null)
                    {
                        sb.AppendLine("SET>" + setMsg);
                    }
                    if (cmdMsg != null)
                    {
                        sb.AppendLine("CMD>" + cmdMsg);
                    }
                    if (check != "0")
                    {
                        sb.AppendLine("CHECK>" + check);
                    }
                    if (sleep != "0")
                    {
                        sb.AppendLine("SLEEP>" + sleep);
                    }
                    if (click != null)
                    {
                        sb.AppendLine("CLICK>" + click);
                    }
                    if (timeout != "0")
                    {
                        sb.AppendLine("TIMEOUT>" + timeout);
                    }
                    if (url != null)
                    {
                        sb.AppendLine("CHECK>url>" + url);
                    }
                }
                foreach (var tstInfo in scriptData.tstInfos)
                {
                    tName = tstInfo.name;
                    tType = tstInfo.type;
                    tDescription = tstInfo.description;
                }
                cmdString = sb.ToString();
                OnStringBuild.Set();
            }
        }

        private bool urlMatch = false;
        private int currentClickTimeout;

        public void DoClickLoop()
        {
            UpdateOutputTextBox($"Executing Clicks At: X={currentClickX}, Y={currentClickY}");
            if (currentClickTimeout != 0)
            {
                double start = DateTimeOffset.Now.ToUnixTimeMilliseconds();
                while (urlMatch == false)
                {
                    MainWindow.AppWindow.MouseClick(currentClickX, currentClickY);
                    
                    double current = DateTimeOffset.Now.ToUnixTimeMilliseconds();
                    if ((current - start) > currentClickTimeout)
                    {
                        UpdateOutputTextBox($"CLICK: X={currentClickX}, Y={currentClickY} - RESULT: Not Found.");

                        logs.Add($"\nCLICK: X={currentClickX}, Y={currentClickY}");
                        logs.Add($"URL: {nextUrl}");
                        logs.Add($"TIMEOUT AT: {(current - startTime) / 1000} sec.");
                        logs.Add("RESULT: Not Found.");
                        logs.Add($"     ERROR (timeout {currentClickTimeout / 1000} sec.)     ");
                        break;
                    }
                    Task.WaitAll(new Task[] { Task.Delay(1000) });
                }
                MainWindow.AppWindow.CheckAddress(false);
                OnClickCheck.Set();
            }
            else
            {
                while (urlMatch == false)
                {
                    MainWindow.AppWindow.MouseClick(currentClickX, currentClickY);
                    Task.WaitAll(new Task[] { Task.Delay(1000) });
                }
                MainWindow.AppWindow.CheckAddress(false);
                OnClickCheck.Set();
            }
        }


        public void OnReceivingClickEvent(object sender, HttpEventArgs e)
        {
            if (e.HttpUrl == nextUrl)
            {
                Debug.WriteLine("\nClick OK");
                UpdateOutputTextBox($"CLICK: X={currentClickX}, Y={currentClickY} - RESULT: Navigation Success.");

                MainWindow.AppWindow.ReceivingHttpStatusCode -= OnReceivingClickEvent;
                urlMatch = true;
                double timestamp = DateTimeOffset.Now.ToUnixTimeMilliseconds();
                logs.Add($"\nCLICK: X={currentClickX}, Y={currentClickY}");
                logs.Add($"URL: {nextUrl}");
                logs.Add($"EXECUTED AT: {(timestamp - startTime) / 1000} sec.");
                logs.Add("RESULT: Navigation Success.");
            }
        }



        public void OnReceivingCoreEvent(object sender, CoreEventArgs e)
        {
            if (nextResult != 0)
            {
                if (e.Code == CoreEventArgs.SuccessCode)
                {
                    currentEventMsg = EventMsg.Success;
                }
                if (e.Code == CoreEventArgs.SpinnerCode)
                {
                    currentEventMsg = EventMsg.Spinner;
                }
                if (e.Code == CoreEventArgs.TimeoutCode)
                {
                    currentEventMsg = EventMsg.Timeout;
                }

                if (e.Code == nextResult)
                {
                    currentResultMsg = ResultMsg.OK;
                    resumeTest = true;
                }
                else
                {
                    currentResultMsg = ResultMsg.NOOK;
                    resumeTest = false;
                }
                SendLogMsg(e.Code);
                OnEventCheck.Set();
            }
        }


        public void OnReceivingHttpEvent(object sender, HttpEventArgs e)
        {
            if (e.HttpStatus == HttpEventArgs.ClientError)
            {
                logs.Add($"\nREQUESTED URL: {e.HttpUrl}");
                logs.Add("RESPONSE CODE: 40X");
                logs.Add("STATUS: Client Error.");
                logs.Add("----------------------------");
                logs.Add("TEST STOP");
                logs.Add("----------------------------");
                MainWindow.AppWindow.ReceivingHttpStatusCode -= OnReceivingHttpEvent;

                UpdateOutputTextBox("HTTP CLIENT ERROR - TEST STOP");
                resumeTest = false;
            }
            if (e.HttpStatus == HttpEventArgs.ServerError)
            {
                logs.Add($"\nREQUESTED URL: {e.HttpUrl}");
                logs.Add("RESPONSE CODE: 50X");
                logs.Add("STATUS: Server Error.");
                logs.Add("----------------------------");
                logs.Add("TEST STOP");
                logs.Add("----------------------------");
                MainWindow.AppWindow.ReceivingHttpStatusCode -= OnReceivingHttpEvent;

                UpdateOutputTextBox("HTTP SERVER ERROR - TEST STOP");
                resumeTest = false;
            }
        }


        private void SendLogMsg(int eCode)
        {
            string eMsg = "", rMsg = "";
            bool failure = false;
            
            switch (currentEventMsg)
            {
                case EventMsg.Success:
                    eMsg = "PING SUCCESS";
                    break;
                case EventMsg.Spinner:
                    eMsg = "SPINNER";
                    break;
                case EventMsg.Timeout:
                    eMsg = "ERROR";
                    break;
            }

            switch (currentResultMsg)
            {
                case ResultMsg.OK:
                    rMsg = "OK";
                    break;
                case ResultMsg.NOOK:
                    rMsg = "NO OK";
                    failure = true;
                    break;
            }

            Debug.WriteLine("----------------------");
            Debug.WriteLine($" CHECK: {eMsg} " + "{CODE " + $"{eCode}" + "}");
            Debug.WriteLine($" Received at: {(DateTimeOffset.Now.ToUnixTimeMilliseconds() - startTime) / 1000}");
            Debug.WriteLine("   Expected: {CODE " + $"{nextResult}" + "}");
            Debug.WriteLine($"     RESULT: {rMsg}     ");
            Debug.WriteLine("----------------------");

            UpdateOutputTextBox($" CHECK: {eMsg} " + "{CODE " + $"{eCode}" + "}" + $" - RESULT: {rMsg}");

            logs.Add("----------------------");
            logs.Add($" CHECK: {eMsg} " + "{CODE " + $"{eCode}" + "}");
            logs.Add($" Received at: {(DateTimeOffset.Now.ToUnixTimeMilliseconds() - startTime) / 1000}");
            logs.Add("   Expected: {CODE " + $"{nextResult}" + "}");
            logs.Add($"     RESULT: {rMsg}     ");
            logs.Add("----------------------");

            if (failure)
            {
                Debug.WriteLine("   END: TEST FAILURE");
                logs.Add("   END: TEST FAILURE");
                UpdateOutputTextBox("   END: TEST FAILURE");
            }
        }


        public void CreateOutputFile()
        {
            UpdateOutputTextBox("CREATING OUTPUTS REPORT");

            Directory.CreateDirectory(Path.Combine(Settings.Default.myTestsPath, "Logs"));
            
            string date = DateTime.Now.ToString("d");
            string time = DateTime.Now.ToString("T");
            string[] lines = logs.ToArray();
            double duration = (endTime - startTime) / 1000;

            string resultPath = Settings.Default.myTestsPath + $@"\Logs\";
            string resultFileName = String.Join("_", "PCM_Test", DateTime.Now.ToString("dd-M-yy"), DateTime.Now.ToString("HH.mm.ss"));
            resultFileName = String.Concat(resultFileName, ".txt");
            string docPath = Path.Combine(resultPath, resultFileName);

            using (StreamWriter outputFile = new StreamWriter(docPath))
            {
                outputFile.WriteLine("DATE: " + $"{date}" + " " + $"{time}");
                outputFile.WriteLine("ID: " + tName);
                outputFile.WriteLine("TYPE: " + tType);
                outputFile.WriteLine("DESCRIPTION: " + tDescription);
                outputFile.WriteLine();
                outputFile.WriteLine("Total execution time: " + duration + " (sec.)");
                outputFile.WriteLine();
                outputFile.WriteLine("TEST LOGS:");

                foreach (string line in lines)
                {
                    outputFile.WriteLine(line);
                }
            }
            Process.Start("notepad.exe", docPath);

            logs.Clear();
        }



























        //public async void RunScriptWithLoop(int duration) // NEW !!!!!
        //{
        //    string filePath = Settings.Default.filePath;
        //    resumeTest = true;
        //    nextResult = 0;
        //    startTime = DateTimeOffset.Now.ToUnixTimeMilliseconds();

        //    DoJsonConversion();
        //    OnStringBuild.WaitOne();

        //    while ((DateTimeOffset.Now.ToUnixTimeMilliseconds() - startTime) < duration && resumeTest)
        //    {
        //        StringReader sr = new StringReader(cmdString);
        //        while (true && resumeTest)
        //        {
        //            string line = sr.ReadLine();
        //            if (line != null)
        //            {
        //                string[] words = line.Split('>');
        //                switch (words[0])
        //                {
        //                    case "CMD":
        //                        {
        //                            if (MainWindow.AppWindow.execOn == "PCM")
        //                            {
        //                                await Task.Run(() => EstablishConnectionToPCM(words[1])); //To use with PCM
        //                            }
        //                            if (MainWindow.AppWindow.execOn == "CMD")
        //                            {
        //                                await Task.Run(() => EstablishConnectionToCMD(words[1])); //To use at home
        //                            }
        //                            break;
        //                        }
        //                    case "CHECK":
        //                        {
        //                            //if (words[1] == "httpcode")
        //                            //{
        //                            //    // TO IMPLEMENT
        //                            //    MainWindow.AppWindow.ReceivingHttpStatusCode += OnReceivingHttpEvent;
        //                            //}
        //                            if (words[1] == "url")
        //                            {
        //                                if (currentClickX != 10000 && currentClickY != 10000)
        //                                {
        //                                    if (words[2] == "")
        //                                    {
        //                                        MainWindow.AppWindow.MouseClick(currentClickX, currentClickY);
        //                                    }
        //                                    else
        //                                    {
        //                                        nextUrl = words[2];
        //                                        MainWindow.AppWindow.CheckAddress(true);
        //                                        MainWindow.AppWindow.ReceivingHttpStatusCode += OnReceivingClickEvent; // PRIMA A SINISTRA AddressChanged
        //                                        DoClickLoop();
        //                                        OnClickCheck.WaitOne();
        //                                    }
        //                                }
        //                            }
        //                            else
        //                            {
        //                                nextResult = Convert.ToInt32(words[1]);
        //                                OnEventCheck.WaitOne();
        //                                nextResult = 0;
        //                            }
        //                            break;
        //                        }
        //                    case "SET":
        //                        {
        //                            if (words[1] == "reconnection")
        //                            {
        //                                CoreLogic.StartConnecting();
        //                            }
        //                            if (words[1] == "localhost")
        //                            {
        //                                Application.Current.Dispatcher.Invoke(() =>
        //                                {
        //                                    MainWindow.AppWindow.addressBar.Text = "http://localhost";
        //                                    MainWindow.AppWindow.SaveSettings();
        //                                });
        //                            }
        //                            if (words[1] == "ip")
        //                            {
        //                                Application.Current.Dispatcher.Invoke(() =>
        //                                {
        //                                    MainWindow.AppWindow.addressBar.Text = words[2];
        //                                    MainWindow.AppWindow.SaveSettings();
        //                                });
        //                            }
        //                            if (words[1] == "vehicleapp")
        //                            {
        //                                Application.Current.Dispatcher.Invoke(() =>
        //                                {
        //                                    MainWindow.AppWindow.VehicleTabItem.IsSelected = true;
        //                                });
        //                            }
        //                            if (words[1] == "httpmonitoring")
        //                            {
        //                                MainWindow.AppWindow.ReceivingHttpStatusCode += OnReceivingHttpEvent;
        //                            }
        //                            break;
        //                        }
        //                    case "SLEEP":
        //                        {
        //                            int pause = Convert.ToInt32(words[1]) * 1000;
        //                            Debug.WriteLine("WAITING FOR " + $"{words[1]}" + " SECONDS...");
        //                            Task.WaitAll(new Task[] { Task.Delay(pause) });
        //                            break;
        //                        }
        //                    case "CLICK":
        //                        {
        //                            string[] coordinates = words[1].Split(',');
        //                            currentClickX = Convert.ToInt32(coordinates[0]);
        //                            currentClickY = Convert.ToInt32(coordinates[1]);
        //                            urlMatch = false;
        //                            break;
        //                        }
        //                    case "TIMEOUT":
        //                        {
        //                            currentClickTimeout = Convert.ToInt32(words[1]) * 1000;
        //                            break;
        //                        }
        //                }
        //            }
        //            else
        //            {
        //                break;
        //            }
        //        }
        //        MainWindow.AppWindow.ReceivingHttpStatusCode -= OnReceivingHttpEvent;
        //        sr.Close();
        //    }
        //    endTime = DateTimeOffset.Now.ToUnixTimeMilliseconds();

        //    CreateOutputFile();
        //}




    }
}
