﻿using System.IO;
using System.Windows;
using System.Windows.Input;
using Microsoft.Win32;
using SimNIS.Properties;
using System.Threading;
using System.Threading.Tasks;
using System;
using Microsoft.WindowsAPICodePack.Dialogs;
using System.Collections.Generic;
using System.Text.Json;
using System.Diagnostics;
using System.Text;
using System.Windows.Controls;
using System.Text.RegularExpressions;

namespace SimNIS
{
    /// <summary>
    /// Logica di interazione per AutomatedTestWindow.xaml
    /// </summary>

    public class Scripts
    {
        public string Name { get; set; }
        public string Type { get; set; }
        public string Description { get; set; }
        public string Path { get; set; }
    }

    public partial class AutomatedTestWindow : Window
    {
        TestLogic TestLogic = TestLogic.Instance;
        public static AutomatedTestWindow TestWindow;
        private int loopDuration = 0;
        private string timeUnit;

        public AutomatedTestWindow()
        {
            InitializeComponent();
            TestWindow = this;
        }


        public void AutomatedTestWindow_Initialized(object sender, EventArgs e)
        {
            TestLogic.RegisterToCoreLogic(true);
            SetCurrentDirectory(true);
            Settings.Default.filePath = "null"; //Initialize file path; prevent unwanted test run
        }


        public async Task StartTest()
        {
            string filePath = Settings.Default.filePath;
            if (filePath != "null")
            {
                if (loopDuration != 0)
                {
                    //TestLogic.RunScriptWithLoop(loopDuration);
                }
                else
                {
                    TestLogic.RunScript();
                }
            }
        }


        private void SetCurrentDirectory(bool defaultPath)
        {
            if (defaultPath)
            {
                if (Settings.Default.myTestsPath == "null")
                {
                    Directory.CreateDirectory(Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), "TestingPCM"));
                    Settings.Default.myTestsPath = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) + $@"\TestingPCM";
                }
            }
            else
            {
                CommonOpenFileDialog dialog = new CommonOpenFileDialog();
                dialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
                dialog.IsFolderPicker = true;
                if (dialog.ShowDialog() == CommonFileDialogResult.Ok)
                {
                    Settings.Default.myTestsPath = dialog.FileName;
                }
            }
            TextBlockPath.Text = Settings.Default.myTestsPath;
            GetCurrentDirectoryFiles();
        }


        public void GetCurrentDirectoryFiles()
        {
            List<Scripts> scripts = new List<Scripts>();

            foreach (string file in Directory.EnumerateFiles(Settings.Default.myTestsPath, "*.json"))
            {
                string fileType = "", fileDescription = "";
                string jsonString = File.ReadAllText(file);
                Script scriptData = JsonSerializer.Deserialize<Script>(jsonString);

                string[] elems = file.Split($@"\");
                int i = 0;
                foreach (var elem in elems)
                {
                    i++;
                }
                string fileName = elems[i - 1];

                foreach (var tstInfo in scriptData.tstInfos)
                {
                    fileName = tstInfo.name;
                    fileType = tstInfo.type;
                    fileDescription = tstInfo.description;
                }
                scripts.Add(new Scripts() { Name = fileName, Type = fileType, Description = fileDescription, Path = file });
            }
            ScriptDataGrid.ItemsSource = scripts;
        }

        private void ScriptDataGrid_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            try
            {
                var row_list = (Scripts)ScriptDataGrid.SelectedItem;
                Settings.Default.filePath = row_list.Path;
            }
            catch { }
        }

        private void ScriptDataGrid_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            var row_list = (Scripts)ScriptDataGrid.SelectedItem;
            new Process
            {
                StartInfo = new ProcessStartInfo(@row_list.Path)
                {
                    UseShellExecute = true
                }
            }.Start();
        }


        private void AddFileToDirectory()
        {
            OpenFileDialog fileDialog = new OpenFileDialog();
            fileDialog.DefaultExt = ".json"; //Required file extension 
            fileDialog.Filter = "Json files (*.json)|*.json"; //Optional file extensions

            bool? res = fileDialog.ShowDialog();
            if (res.HasValue && res.Value)
            {
                string source = fileDialog.FileName;
                string dest = Settings.Default.myTestsPath + @"\" + Path.GetFileName(source);
                File.Copy(source, dest);
            }
            GetCurrentDirectoryFiles();
        }


        private void SelectFile() //OLD: This method was used to select the script to run, before implementing the direct selection in Datagrid.
        {
            OpenFileDialog fileDialog = new OpenFileDialog();
            fileDialog.DefaultExt = ".json"; // Required file extension
            fileDialog.Filter = "Json files (*.json)|*.json"; // Optional file extensions

            bool? res = fileDialog.ShowDialog();
            if (res.HasValue && res.Value)
            {
                //TextBlockFileName.Text = fileDialog.FileName;
                Settings.Default.filePath = fileDialog.FileName;
            }
        }


        private void TimeUnitComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ComboBoxItem timeUnitItem = (ComboBoxItem)TimeUnitComboBox.SelectedItem;
            if (LoopDurationTextBox != null)
            {
                if (timeUnitItem == SecondsUnit)
                {
                    timeUnit = "sec";
                }
                if (timeUnitItem == MinutesUnit)
                {
                    timeUnit = "min";
                }
            }
        }

        private void LoopDurationTextBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (LoopDurationTextBox.Text != "")
            {
                switch (timeUnit)
                {
                    case "sec":
                        {
                            loopDuration = Convert.ToInt32(LoopDurationTextBox.Text) * 1000;
                            break;
                        }
                    case "min":
                        {
                            loopDuration = Convert.ToInt32(LoopDurationTextBox.Text) * 60000;
                            break;
                        }
                }
            }
        }



        /*UI code*/

        private void Window_MouseDown(object sender, MouseButtonEventArgs e)
        {
            if (e.ChangedButton == MouseButton.Left && e.ClickCount != 2)
                DragMove();
        }


        private void ButtonMinimizeTestWindow_Click(object sender, RoutedEventArgs e)
        {
            this.WindowState = WindowState.Minimized;
            MainWindow.AppWindow.WindowState = WindowState.Minimized;
        }


        public void ButtonCloseTestWindow_Click(object sender, RoutedEventArgs e)
        {
            TestLogic.RegisterToCoreLogic(false);
            Close();
        }


        private void ButtonInfo_Click(object sender, RoutedEventArgs e)
        {
            new Thread(() => MessageBox.Show("The directory where all the scripts for the available tests listed below are stored." + "\n" + $@"By default, it is set to C:\...\Documents\TestingPCM" + "\n\n" + "- To modify the current directory, click on gear icon.\n" + "- To update all the available tests in the current directory, click on refresh icon.\n" + "- To load a new test script and save it in the current directory, click on new file icon.\n")).Start();
        }


        private void ButtonSetPath_Click(object sender, RoutedEventArgs e)
        {
            SetCurrentDirectory(false);
        }


        private void ButtonRefreshPath_Click(object sender, RoutedEventArgs e)
        {
            GetCurrentDirectoryFiles();
        }


        private void ButtonAddFile_Click(object sender, RoutedEventArgs e)
        {
            AddFileToDirectory();
        }


        private void ButtonBrowseScript_Click(object sender, RoutedEventArgs e)
        {
            SelectFile();
        }


        private async void ButtonRunTest_Click(object sender, RoutedEventArgs e)
        {
            OutputTextBox.Text = "";
            await Task.Run(() => StartTest());
        }


        private void ButtonRecordInputs_Click(object sender, RoutedEventArgs e)
        {
            ButtonRecordInputs.Visibility = Visibility.Hidden;
            ButtonStopRecord.Visibility = Visibility.Visible;
            Application.Current.Dispatcher.Invoke(() =>
            {
                MainWindow.AppWindow.VehicleTabItem.IsSelected = true;
            });
            MainWindow.AppWindow.DoRecording(true);
        }


        private void ButtonStopRecord_Click(object sender, RoutedEventArgs e)
        {
            ButtonRecordInputs.Visibility = Visibility.Visible;
            ButtonStopRecord.Visibility = Visibility.Hidden;
            MainWindow.AppWindow.DoRecording(false);
        }

        private void LoopDurationTextBox_PreviewTextInput(object sender, TextCompositionEventArgs e)
        {
            Regex regex = new Regex("[^0-9]+");
            e.Handled = regex.IsMatch(e.Text);
        }
    }
}
