﻿using System;
using System.Diagnostics;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Text.RegularExpressions;
using SimNIS.Properties;
using System.IO;
using System.Threading;
using CefSharp.Wpf;
using CefSharp;
using CefSharp.OffScreen;
using HtmlAgilityPack;
using System.Net;
using System.Collections.Generic;
using CefSharp.Handler;
using System.Runtime.InteropServices;

namespace SimNIS
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>

    public class HttpEventArgs : EventArgs
    {
        public int HttpStatus { get; set; }
        public const int RedirectionMsg = 3;
        public const int ClientError = 4;
        public const int ServerError = 5;
        public string HttpUrl { get; set; }
    }

    public partial class MainWindow : Window
    {
        private enum VehicleType {Heavy, Light}
        private VehicleType currentVehicleType;
        UserControl SpinnerControl = new SpinnerControl();
        CoreLogic CoreLogic = CoreLogic.Instance; 
        public static MainWindow AppWindow;
        public string homepage, execOn;
        private CefSharp.Wpf.ChromiumWebBrowser browser;
        private CefSharp.OffScreen.ChromiumWebBrowser debugBrowser;



        public MainWindow()
        {
            InitializeComponent();
            AppWindow = this;

            CefSharp.Wpf.CefSettings settings = new CefSharp.Wpf.CefSettings()
            {
                CachePath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), "CefSharp\\Cache"),
                UserAgent = "Mozilla/5.0 (Linux; U; Android 4.4.2; en-us; SCH-I535 Build/KOT49H) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30",
                RemoteDebuggingPort = 8088
            };
            Cef.Initialize(settings);

            InitializeBrowser();
        }


        private async void InfoTab_Initialized(object sender, EventArgs e)
        {
            OnStartup();
            await Task.Run(() => MainLogic());
        }


        public async Task MainLogic()
        {
            CoreLogic.PingSuccess += OnPingSuccess;
            CoreLogic.SpinnerLaunch += OnSpinnerLaunch;
            CoreLogic.ErrorTimeout += OnErrorTimeout;
            CoreLogic.InitializeLogic();
        }


        public void OnPingSuccess(object source, EventArgs e)
        {
            Application.Current.Dispatcher.Invoke(() =>
            {
                Debug.WriteLine(" ___REQUESTING HOMEPAGE___");
                browser.Load(homepage);
            });
        }


        public void OnSpinnerLaunch(object source, EventArgs e)
        {
            Application.Current.Dispatcher.Invoke(() =>
            {
                Debug.WriteLine("___LAUNCHING SPINNER ANIMATION___");
                SetPage(true, false, false);
            });
        }


        public void OnErrorTimeout(object source, EventArgs e)
        {
            Application.Current.Dispatcher.Invoke(() =>
            {
                Debug.WriteLine("___DISPLAYING ERROR PAGE___");
                SetPage(false, false, true);
            });
        }


        private void InitializeSettings()
        {
            if (execOn == "PCM")
            {
                Debug.WriteLine("Setting configuration for PCM...\n");
                addressBar.Text = Settings.Default.addressPCM;
                homepage = Settings.Default.homepagePCM;
                homepageBar.Text = homepage;
                timeoutBar.Text = Convert.ToString(Settings.Default.timeoutPCM);

            }
            if (execOn == "CMD")
            {
                Debug.WriteLine("Setting configuration for local server...\n");
                addressBar.Text = Settings.Default.addressCMD;
                homepage = Settings.Default.homepageCMD;
                homepageBar.Text = homepage;
                timeoutBar.Text = Convert.ToString(Settings.Default.timeoutCMD);
            }
            CoreLogic.address = addressBar.Text;
            CoreLogic.timeout = Convert.ToInt32(timeoutBar.Text);
        }


        public void SaveSettings()
        {
            Debug.WriteLine("\nSaving and applying settings...");
            switch (execOn)
            {
                case "CMD":
                    {
                        Settings.Default.addressCMD = addressBar.Text;
                        Settings.Default.timeoutCMD = Convert.ToInt32(timeoutBar.Text);
                        Settings.Default.homepageCMD = homepageBar.Text;
                        break;
                    }
                case "PCM":
                    {
                        Settings.Default.addressPCM = addressBar.Text;
                        Settings.Default.timeoutPCM = Convert.ToInt32(timeoutBar.Text);
                        Settings.Default.homepagePCM = homepageBar.Text;
                        break;
                    }
            }
            Settings.Default.Save();
            CoreLogic.address = addressBar.Text;
            CoreLogic.timeout = Convert.ToInt32(timeoutBar.Text);
            homepage = homepageBar.Text;
            if (CoreLogic.currentStatus == CoreLogic.Status.Success)
            {
                browser.Load(homepage);
            }
        }


        private async void SetMode()
        {
            string args;
            if (debugModeCheckBox.IsChecked == true)
            {
                Debug.WriteLine("\nDebug Mode: ENABLED");
                execOn = "CMD";
                args = Directory.GetCurrentDirectory() + $@"\www\bin\tiny.exe " + Directory.GetCurrentDirectory() + $@"\www\root";
            }
            else
            {
                Debug.WriteLine("\nDebug Mode: DISABLED");
                execOn = "PCM";
                args = $@"taskkill /F /IM tiny.exe";
            }
            InitializeSettings();
            await Task.Run(() => RequestWebServer(args));
        }


        private void OnExit()
        {
            switch (execOn)
            {
                case "CMD":
                    {
                        Settings.Default.lastModeIsDebug = true;
                        break;
                    }
                case "PCM":
                    {
                        Settings.Default.lastModeIsDebug = false;
                        break;
                    }
            }
            Settings.Default.lastTab = InfoTab.SelectedIndex;
            Settings.Default.Save();
        }


        private void OnStartup()
        {
            InfoTab.SelectedIndex = Settings.Default.lastTab;

            if (Settings.Default.lastModeIsDebug)
            {
                debugModeCheckBox.IsChecked = true;
            }
            else
            {
                debugModeCheckBox.IsChecked = false;
                SetMode();
            }
        }


        public async Task RequestWebServer(string args)
        {
            ProcessStartInfo cmd = new ProcessStartInfo();
            cmd.UseShellExecute = true;
            cmd.WorkingDirectory = @"C:\Windows\System32";
            cmd.FileName = @"C:\Windows\System32\cmd.exe";
            cmd.Arguments = "/c " + args;
            cmd.WindowStyle = ProcessWindowStyle.Hidden;
            Process process = Process.Start(cmd);
            process.WaitForExit();

            if (process.HasExited)
            {
                process.Close();
                process.Dispose();
            }
        }





        /*UI code*/


        public void Window_MouseDown(object sender, MouseButtonEventArgs e)
        {
            if (e.ChangedButton == MouseButton.Left)
                DragMove();
        }


        private void ButtonMinimizeAppWindow_Click(object sender, RoutedEventArgs e)
        {
            this.WindowState = WindowState.Minimized;
        }


        private void ButtonCloseAppWindow_Click(object sender, RoutedEventArgs e)
        {
            OnExit();
            Application.Current.Shutdown();
        }


        private void HeaderSWay_Click(object sender, RoutedEventArgs e)
        {
            currentVehicleType = VehicleType.Heavy;

            SetMenuItemHeaderSWay(sender as MenuItem);
            SetVehicleTabIcon();
        }


        private void HeaderDaily_Click(object sender, RoutedEventArgs e)
        {
            currentVehicleType = VehicleType.Light;

            SetMenuItemHeaderDaily(sender as MenuItem);
            SetVehicleTabIcon();
        }


        private void SetMenuItemHeaderSWay(MenuItem menuItem)
        {
            var sb = new StringBuilder();

            if (menuItem != null)
            {
                headerVehicleModel.Header = "";
                headerVehicleModel.Header = sb.Append(headerVehicleModel.Header).Append("S-WAY ").Append(menuItem.Header);
            }
        }


        private void SetMenuItemHeaderDaily(MenuItem menuItem)
        {
            var sb = new StringBuilder();

            if (menuItem != null)
            {
                headerVehicleModel.Header = "";
                headerVehicleModel.Header = sb.Append(headerVehicleModel.Header).Append("DAILY ").Append(menuItem.Header);
            }
        }


        private void SetVehicleTabIcon()
        {
            HeavyTabItemIcon.Visibility = Visibility.Collapsed;
            LightTabItemIcon.Visibility = Visibility.Collapsed;

            if (currentVehicleType == VehicleType.Heavy)
            {
                HeavyTabItemIcon.Visibility = Visibility.Visible;
            }   
            if (currentVehicleType == VehicleType.Light)
            {
                LightTabItemIcon.Visibility = Visibility.Visible;
            }
        }


        private void SetPage(bool setSpinnerControl, bool setBrowser, bool setPageNotFound)
        {
            SpinnerControl.Visibility = Visibility.Collapsed;
            BrowserGrid.Visibility = Visibility.Hidden;
            PageNotFound.Visibility = Visibility.Collapsed;
            HeavyNotFound.Visibility = Visibility.Collapsed;
            LightNotFound.Visibility = Visibility.Collapsed;

            if (setSpinnerControl)
            {
                SpinnerControl.Visibility = Visibility.Visible;
            }
            if (setBrowser)
            {
                BrowserGrid.Visibility = Visibility.Visible;
            }
            if (setPageNotFound)
            {
                PageNotFound.Visibility = Visibility.Visible;
                if (currentVehicleType == VehicleType.Heavy)
                {
                    HeavyNotFound.Visibility = Visibility.Visible;
                }
                if (currentVehicleType == VehicleType.Light)
                {
                    LightNotFound.Visibility = Visibility.Visible;
                }
            }
        }


        private void ButtonBack_Click(object sender, RoutedEventArgs e)
        {
            CoreLogic.StartConnecting();
        }


        private void TimeoutBar_PreviewTextInput(object sender, TextCompositionEventArgs e)
        {
            Regex regex = new Regex("[^0-9]+");
            e.Handled = regex.IsMatch(e.Text);
        }

        
        private void ButtonSave_Click(object sender, RoutedEventArgs e)
        {
            SaveSettings();
        }
        

        private void ButtonOpenTestWindow_Click(object sender, RoutedEventArgs e)
        {
            var testWindow = new AutomatedTestWindow();
            testWindow.Show();
        }


        private void debugModeCheckBox_CheckedChanged(object sender, RoutedEventArgs e)
        {
            SetMode();
            CoreLogic.StartConnecting();
        }


        





        /* BROWSER IMPLEMENTATION */



        public void InitializeBrowser()
        {
            Debug.WriteLine("\nINITIALIZING BROWSER");
            browser = new CefSharp.Wpf.ChromiumWebBrowser();
            browser.RequestHandler = new CustomRequestHandler();
            BrowserGrid.Children.Add(browser);
            
            debugBrowser = new CefSharp.OffScreen.ChromiumWebBrowser();

            browser.FrameLoadEnd += Browser_FrameLoadEnd;
            debugBrowser.FrameLoadEnd += DebugBrowser_FrameLoadEnd;
        }


        private void Browser_FrameLoadEnd(object sender, FrameLoadEndEventArgs e)
        {
            if (execOn == "PCM")
            {
                Dispatcher.BeginInvoke((Action)(() =>
                {
                    Debug.WriteLine("       1        ");
                    browser.SetZoomLevel(-2.58);
                    debugBrowser.Load("localhost:8088");
                }));
            }
            if (execOn == "CMD")
            {
                Dispatcher.Invoke(() =>
                {
                    Debug.WriteLine("DISPLAYING HOMEPAGE.\n");
                    SetPage(false, true, false);
                });
            }
        }


        private void DebugBrowser_FrameLoadEnd(object sender, FrameLoadEndEventArgs e)
        {
            Dispatcher.BeginInvoke((Action)(async () =>
            {
                Debug.WriteLine("       2        ");
                string html = await debugBrowser.GetSourceAsync();
                TextReader tr = new StringReader(html);
                DoHtmlParse(tr);
            }));
        }


        private bool debugBrowserActive = false;

        public void DoHtmlParse(TextReader html)
        {
            Debug.WriteLine("       3        ");
            HtmlDocument htmlDoc = new HtmlDocument();
            htmlDoc.Load(html);
            try
            {
                string result = htmlDoc.DocumentNode.SelectSingleNode("//body/div[2]/p/a").GetAttributeValue("href", "");
                StringBuilder sb = new StringBuilder("localhost:8088");
                string debugUrl = sb.Append(result).ToString();
                new CefSharp.OffScreen.ChromiumWebBrowser(debugUrl);

                debugBrowserActive = true;
            }
            catch (Exception)
            {
                if (debugBrowserActive == false)
                {
                    Debug.WriteLine("No remote session.\n");
                    MessageBox.Show("Please reconnect PCM and restart the app.");
                }
            }
            Debug.WriteLine("DISPLAYING HOMEPAGE.\n");
            SetPage(false, true, false);
        }


        
        public string currentClickUrl;

        private void Browser_AddressChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            Debug.WriteLine("\nADDRESS CHANGED");
            if (ReceivingHttpStatusCode != null)
            {
                ReceivingHttpStatusCode(this, new HttpEventArgs() { HttpUrl = e.NewValue.ToString() });
            }
            currentClickUrl = e.NewValue.ToString();
        }








        /* HTTP STATUS CODES */



        public event EventHandler<HttpEventArgs> ReceivingHttpStatusCode;

        protected virtual void OnRedirectionMsg(string requestUrl)
        {
            if (ReceivingHttpStatusCode != null)
            {
                ReceivingHttpStatusCode(this, new HttpEventArgs() { HttpStatus = HttpEventArgs.RedirectionMsg , HttpUrl = requestUrl });
            }
        }

        protected virtual void OnClientError(string requestUrl)
        {
            if (ReceivingHttpStatusCode != null)
            {
                ReceivingHttpStatusCode(this, new HttpEventArgs() { HttpStatus = HttpEventArgs.ClientError , HttpUrl = requestUrl });
            }
        }

        protected virtual void OnServerError(string requestUrl)
        {
            if (ReceivingHttpStatusCode != null)
            {
                ReceivingHttpStatusCode(this, new HttpEventArgs() { HttpStatus = HttpEventArgs.ServerError , HttpUrl = requestUrl });
            }
        }

        public class CustomRequestHandler : RequestHandler
        {
            protected override IResourceRequestHandler GetResourceRequestHandler(IWebBrowser chromiumWebBrowser, IBrowser browser, IFrame frame, IRequest request, bool isNavigation, bool isDownload, string requestInitiator, ref bool disableDefaultHandling)
            {
                return new CustomResourceRequestHandler();
            }
        }


        public class CustomResourceRequestHandler : ResourceRequestHandler
        {
            protected override bool OnResourceResponse(IWebBrowser chromiumWebBrowser, IBrowser browser, IFrame frame, IRequest request, IResponse response)
            {
                if (response.StatusCode != (int)HttpStatusCode.OK)
                {
                    if (response.StatusCode == (int)HttpStatusCode.NotModified)
                    {
                        Debug.WriteLine($"\nHttpCode: {response.StatusCode} - Msg: Client's cached copy is up to date");
                        MainWindow.AppWindow.OnRedirectionMsg(request.Url);
                    }
                    if (response.StatusCode >= 400 && response.StatusCode <= 499)
                    {
                        Debug.WriteLine($"\nHttpCode: {response.StatusCode} - Url: {request.Url}");
                        MainWindow.AppWindow.OnClientError(request.Url);
                    }
                    if (response.StatusCode >= 500 && response.StatusCode <= 599)
                    {
                        Debug.WriteLine($"\nHttpCode: {response.StatusCode} - Url: {request.Url}");
                        MainWindow.AppWindow.OnServerError(request.Url);
                    }
                }
                return false; //Allow the resource to load normally
            }
        }






        /* INPUTS RECORDING */

        List<string> clicks = new List<string>();
        int clickCount;

        private void Browser_MouseUp(object sender, MouseButtonEventArgs e)
        {
            Point position = e.GetPosition((IInputElement)sender);
            int posX = (int)position.X;
            int posY = (int)position.Y;
            Debug.WriteLine($"RECORD: X={posX} Y={posY}");
            char c = '"';
            char p1 = '{';
            char p2 = '}';
            string id = $"{c}id{c}: {c}Click{clickCount}{c}";
            string click = $"{c}click{c}: {c}{posX},{posY}{c}";
            string url = $"{c}url{c}: {c}{currentClickUrl}{c}";
            string sleep = $"{c}sleep{c}: 3";
            string timeout = $"{c}timeout{c}: 5";
            clicks.Add(p1.ToString());
            clicks.Add(id);
            clicks.Add(click);
            clicks.Add(timeout);
            clicks.Add(url);
            clicks.Add(sleep);
            clicks.Add(p2.ToString());
            clickCount++;
        }


        public void DoRecording(bool enabled)
        {
            if (enabled)
            {
                browser.AddHandler(MouseUpEvent, new MouseButtonEventHandler(Browser_MouseUp), true);
                CheckAddress(true);
                clickCount = 1;
            }
            else
            {
                browser.MouseUp -= Browser_MouseUp;
                CheckAddress(false);
                CreateRecordInputsFile(clicks);
            }
        }

        public void CheckAddress(bool enabled)
        {
            if (enabled)
                browser.AddressChanged += Browser_AddressChanged;
            else
                browser.AddressChanged -= Browser_AddressChanged;
        }

        public void CreateRecordInputsFile(List<string> clicks)
        {
            Directory.CreateDirectory(Path.Combine(Settings.Default.myTestsPath, "Records"));
            string[] lines = clicks.ToArray();
            string resultPath = Settings.Default.myTestsPath + $@"\Records\";
            string resultFileName = String.Join("_", "RecInputs", DateTime.Now.ToString("dd-M-yy"), DateTime.Now.ToString("HH.mm.ss"));
            resultFileName = String.Concat(resultFileName, ".txt");
            string docPath = Path.Combine(resultPath, resultFileName);

            using (StreamWriter outputFile = new StreamWriter(docPath))
            {
                foreach (string line in lines)
                {
                    outputFile.WriteLine(line);
                }
            }
            Process.Start("notepad.exe", docPath);
        }


        public void MouseClick(int x, int y)
        {
            Debug.WriteLine($"MouseClick: X={x} Y={y}");
            Dispatcher.Invoke(() =>
            {
                RootCanvas.Visibility = Visibility.Visible;
                Canvas.SetTop(customPointer, y);
                Canvas.SetLeft(customPointer, x);
            });
            browser.GetBrowser().GetHost().SendMouseClickEvent(x, y, MouseButtonType.Left, false, 1, CefEventFlags.None);
            Thread.Sleep(15);
            browser.GetBrowser().GetHost().SendMouseClickEvent(x, y, MouseButtonType.Left, true, 1, CefEventFlags.None);
            Task.Delay(2000).ContinueWith(_ =>
                {
                    Dispatcher.Invoke(() =>
                    {
                        RootCanvas.Visibility = Visibility.Collapsed;
                    });
                }
            );
        }

        


    }
}

